/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

/**********************************************************************
 resources.c

 Turns a Netscape.ad file into a netscape.cfg file so we can do
 l10n for Dogbert.

 Usage:
		resources Netscape.ad
**********************************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include "md5.h"


#define OUT_FILE_NAME "netscape.cfg"
#define COUNT(a) (sizeof(a)/sizeof((a)[0]))


char output_buf[1024 * 1024];
char file_buf[1024 * 1024];


/*
 * usage
 */
void
usage(char* prog_name)
{
    fprintf(stderr, "Usage: %s filename\n", prog_name);
}


/*
 * append_to_file
 */
void
append_to_file(char buf[])
{
    strcat(output_buf, buf);
}

/*
 * append_to_buf
 */
void
append_to_buf(char buf[])
{
    strcat(file_buf, buf);
}

/*
 * obscure
 */
static void
obscure(char* buf)
{
    int i;
    static int offset = 0;
    int len = buf ? strlen(buf) : 0;
 
    for ( i = 0; i < len; i++, offset++ ) {
        buf[i]+= 7;
    }
}


/*
 * write_hash
 */
static void
write_hash(FILE* file)
{
    char buf[256];
    unsigned char digest[16];
    unsigned char magic_key[] = "VonGloda5652TX75235ISBN";
    int len;
 
    MD5_CTX md5_cxt;
    MD5Init(&md5_cxt);
 
    /* start with the magic key */
    MD5Update(&md5_cxt, magic_key, sizeof(magic_key));
 
    MD5Update(&md5_cxt, output_buf, strlen(output_buf));

    fprintf(stderr, "%d bytes\n", strlen(output_buf));

    MD5Final(digest, &md5_cxt);
 
    sprintf(buf, "// %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
            digest[0], digest[1], digest[2], digest[3],
            digest[4], digest[5], digest[6], digest[7],
            digest[8], digest[9], digest[10], digest[11],
            digest[12], digest[13], digest[14], digest[15]);
 
    obscure(buf);
 
    if ( fwrite(buf, sizeof(char), strlen(buf), file) != strlen(buf) ) {
        perror("fwrite");
        exit(errno);
    }
}


/*
 * output_file
 */
void
output_file(FILE* out)
{
    int i, j, k;
    int len;
    char buf[256];

    append_to_file("with ( PrefConfig ) {\n");

    append_to_file(file_buf);

    append_to_file("}\n");
    write_hash(out);
    len = strlen(output_buf);
    obscure(output_buf);
    fwrite(output_buf, sizeof(char), len, out);
}


/*
 * fatal
 */
void
fatal(char* progname, int error)
{
	fprintf(stderr, "%s failed\n", progname);
	exit(error);
}


/*
 * main
 */
int
main(int argc, char* argv[])
{
    FILE* in;
    FILE* out;
    char buf[64 * 1024];
	int line = 0;

    if ( argc != 2 ) {
        usage(argv[0]);
        fatal(argv[0], 1);
    }

    if ( (in = fopen(argv[1], "r")) == NULL ) {
        perror(argv[1]);
        fatal(argv[0], errno);
    }

    if ( (out = fopen(OUT_FILE_NAME, "w")) == NULL ) {
        perror(OUT_FILE_NAME);
        fatal(argv[0], errno);
    }

    while ( fgets(buf, sizeof(buf), in) != NULL ) {
		line++;
		if ( strchr(buf, '\n') == NULL ) {
			fprintf(stderr, "%s, line %d: Line too long\n", argv[1], line);
			fatal(argv[0], 1);
		}
        append_to_buf(buf);
    }

    output_file(out);

    if ( fclose(in) != 0 ) {
        perror(argv[1]);
        fatal(argv[0], errno);
    }

    if ( fclose(out) != 0 ) {
        perror(OUT_FILE_NAME);
        fatal(argv[0], errno);
    }

	fprintf(stderr, "Created " OUT_FILE_NAME "\n");

    return 0;
}
